@@ -29,8 +29,9 @@ import traceback
 
 import _dbus_bindings
 from dbus import SessionBus
-from dbus.exceptions import NameExistsException
-from dbus.exceptions import UnknownMethodException
+from dbus.exceptions import DBusException, \
+                            NameExistsException, \
+                            UnknownMethodException
 from dbus.decorators import method
 from dbus.decorators import signal
 from dbus.proxies import LOCAL_PATH
@@ -249,8 +250,10 @@ def _method_reply_return(connection, message, method_name, signature, *retval):
 
 
 def _method_reply_error(connection, message, exception):
-    if hasattr(exception, '_dbus_error_name'):
-        name = exception._dbus_error_name
+    name = getattr(exception, '_dbus_error_name', None)
+
+    if name is not None:
+        pass
     elif getattr(exception, '__module__', '') in ('', '__main__'):
         name = 'org.freedesktop.DBus.Python.%s' % exception.__class__.__name__
     else:
@@ -377,7 +380,7 @@ class Object(Interface):
         is also required.
 
         :Parameters:
-            `conn` : dbus.Connection
+            `conn` : dbus.connection.Connection
                 The connection on which to export this object.
 
                 If None, use the Bus associated with the given ``bus_name``,
@@ -423,6 +426,39 @@ class Object(Interface):
     __dbus_object_path__ = property(lambda self: self._object_path, None, None,
                                     "The D-Bus object path of this object")
 
+    def remove_from_connection(self, connection=None, path=None):
+        """Make this object inaccessible via the given D-Bus connection
+        and object path. If no connection or path is specified,
+        the object ceases to be accessible via any connection or path.
+
+        It's not currently possible to export an object on more than one
+        connection or with more than one object-path, but this will be
+        supported in future.
+
+        :Parameters:
+            `connection` : dbus.connection.Connection or None
+                Only remove the object from this Connection. If None,
+                remove from all Connections on which it's exported.
+            `path` : dbus.ObjectPath or other str, or None
+                Only remove the object from this object path. If None,
+                remove from all object paths.
+        :Raises LookupError:
+            if the object was not exported on the requested connection
+            or path, or (if both are None) was not exported at all.
+        """
+        if self._object_path is None or self._connection is None:
+            raise LookupError('%r is not exported' % self)
+        if path is not None and self._object_path != path:
+            raise LookupError('%r is not exported at path %r' % (self, path))
+        if connection is not None and self._connection != connection:
+            raise LookupError('%r is not exported on %r' % (self, connection))
+
+        try:
+            self._connection._unregister_object_path(self._object_path)
+        finally:
+            self._connection = None
+            self._object_path = None
+
     def _unregister_cb(self, connection):
         _logger.info('Unregistering exported object %r', self)
 
@@ -527,6 +563,10 @@ class Object(Interface):
 
             reflection_data += '  </interface>\n'
 
+        for name in self._connection.list_exported_child_objects(
+                self._object_path):
+            reflection_data += '  <node name="%s"/>\n' % name
+
         reflection_data += '</node>\n'
 
         return reflection_data
