@@ -68,6 +68,8 @@ class TestDBusBindings(unittest.TestCase):
     def setUp(self):
         self.bus = dbus.SessionBus()
         self.remote_object = self.bus.get_object(NAME, OBJECT)
+        self.remote_object_follow = self.bus.get_object(NAME, OBJECT,
+                follow_name_owner_changes=True)
         self.iface = dbus.Interface(self.remote_object, IFACE)
 
     def testGObject(self):
@@ -383,6 +385,29 @@ class TestDBusBindings(unittest.TestCase):
         self.assert_(not isinstance(ret, dbus.Struct), repr(ret))
         self.assertEquals(ret, ('abc', 123))
 
+    def testListExportedChildObjects(self):
+        self.assert_(self.iface.TestListExportedChildObjects())
+
+    def testRemoveFromConnection(self):
+        # https://bugs.freedesktop.org/show_bug.cgi?id=10457
+        self.assert_(not self.iface.HasRemovableObject())
+        self.assert_(self.iface.AddRemovableObject())
+        self.assert_(self.iface.HasRemovableObject())
+
+        removable = self.bus.get_object(NAME, OBJECT + '/RemovableObject')
+        iface = dbus.Interface(removable, IFACE)
+        self.assert_(iface.IsThere())
+        self.assert_(iface.RemoveSelf())
+
+        self.assert_(not self.iface.HasRemovableObject())
+
+        # and again...
+        self.assert_(self.iface.AddRemovableObject())
+        self.assert_(self.iface.HasRemovableObject())
+        self.assert_(iface.IsThere())
+        self.assert_(iface.RemoveSelf())
+        self.assert_(not self.iface.HasRemovableObject())
+
 """ Remove this for now
 class TestDBusPythonToGLibBindings(unittest.TestCase):
     def setUp(self):
